package BK::ServFunc::EditFunc;

require 5.002;
use BK::Composer;
use POSIX;

sub register {
    my ($books) = shift;

    opendir(BOOKS, $books) || die "Can't open $books: $!";
    my %cmds;
    foreach $db (map { chop;chop;chop;chop; "\l$_"; }
                        grep {/.*\.rdb/} readdir(BOOKS)) {
	# Make the edit and update functions.
	$cmd = BK::ServFunc::EditFunc::newFunc($db);
	$cmds{"user-$db"} = $cmd;

	$cmd = BK::ServFunc::EditFunc::newUpdateFunc($db);
	$cmds{"server-Update\u$db"} = $cmd;
    }
    if (not %cmds) {
	print "WARNING: unable to find any RDB tables.  ",
		"Consider specifying a directory.\n";
    }

    closedir(BOOKS);
    return \%cmds;
}

sub newUpdateFunc {
    my $base = shift;
    return { func => sub {
    			BK::ServFunc::EditFunc::update("\u$base", @_);
		     },
	};
}

sub newFunc {
    my $base = shift;
    my $base3 = substr($base, 0, 3);
    my $a = 'a';

    $a = 'an' if $base =~ /^[aeiou]/;
    
    return { shorthelp => "Create, delete, or edit $a $base",
	     func => sub { BK::ServFunc::EditFunc::run("\u$base", @_); },
	     help => <<"__EOH__" };

    $base -- Create, delete, or edit $a $base

    bk $base [ -t template ] [ ID ]
    bk $base -o [ -t template ] [ ID ]
    bk $base -d ID
    bk $base -i

        '\u$base' creates and edits $base values using an ASCII form.
        \u$a $base is an entry in the bookkeeping system.

        With no arguments, '$base' creates a blank $base form and
        invokes the user's editor.  When the form is saved, $a $base of the
        form ${base3}NNNNNN is created.  If an ID is given on the command line
        either that named ID will be created or, if the ID already exists,
        the $base can be modified.  Use the -t option to indicate an
        existing entry that will supply initial values.

        The -d flag deletes the named $base.
        The -o flag causes '$base' to write the $base form to the
        standard output, instead of invoking the editor.  The -i flag causes
        '$base' to read $a $base form from the standard input.
__EOH__
}

sub run {
    my ($name, $server, $table, $args) = @_;
    my ($arg, $templateID) = (undef, undef);

    # Strip out '-t template' if set.

    $_ = join(' ', @$args);
    if (/-t\s+(\S+)/) {
	$templateID = $1;
	s/-t\s+(\S+)\s*//;
	@$args = split ' ';
    }


    # Now do the sub-subcommand.

    push(@$args, "new") if (not scalar(@$args));
    $_ = shift(@$args);
    if (/^[^-]/) {
        $server->editEntry($name, $_, $templateID);
    } else {
        if (/-d/) {
            die "No \l$name ID specified." if not @$args;
            $server->deleteEntry($name, shift(@$args));
        } elsif (/-i/) {
            $server->readEntry($name);
        } elsif (/-o/) {
            $server->writeEntry($name,
                                @$args ? shift(@$args) : 'new',
                                $templateID);
        } else {
	    die "Unknown option \"$_\".  Try 'bk help \l$name'.";
	}
    }

}

sub update {
    my ($name, $server, $table, $args) = @_;
    my %entry = parseData($server, $name, $table->{data});

    my $state = 'created';
    if ($entry{$name} eq 'new') {
        $entry{$name} = $server->newID($name);
    } else {
        $server->reallyDelete($name, $entry{$name});
	$state = 'saved';
    }

    my @fields = @{($server->getHeader($name))[0]};
    shift(@fields);
    my $entry = $entry{$name};
    foreach (@fields) {
        $entry .= "\t" . $entry{$_};
    }

    my $db = $server->home() . "/" . $name . ".rdb";
    # Untaint $db.
    $db =~ /(.*)/; $db = $1;

    open(DATABASE, ">>$db") || die "Can't open $db: $!\n";
    print DATABASE $entry, "\n";
    close DATABASE;

    # print "$name '$entry{$name}' $state.\n";
    print {$server->fd()} &generate({
                     data => "$name '$entry{$name}' $state.",
                     func => 'client-OutputInfo',
               });
    $server->release();
}

sub parseData {
    my ($server, $name, $data) = @_;
    my ($f, $t, $d) = $server->getHeader($name);

    my ($num, %indexes) = (0, ());
    for (@$f) { $indices{$_} = $num++; }

    my %entry = ();
    my @lines = split /\n/, $data;
    while (@lines) {
        $_ = shift(@lines);
        next if (/^#/);          # comment;
        next if (!/^\w+:/);      # No key/value pair
	next if (!length);       # a blank line;

        my ($field, $val) = split(/:/, $_, 2);
        $val =~ s/\s*//;
        $val =~ s/\s*$//;

        if ($d->[$indices{$field}] =~ /@@.*MULTILINE/) {
            while (@lines) {
                $_ = shift(@lines);
                unshift(@lines, $_), last if (/^\w+:/);
                $val .= '\N' . $_;
            }
            $val =~ s/(\\N)*$//;
        }
        $val =~ s/\t/\\T/g;
        $entry{$field} = $val;
    }

    while (@$f && @$t && @$d) {
        ($field, $type, $desc) = (shift @$f, shift @$t, shift @$d);
        $server->verify($entry{$field}, $field, $type, $desc);
    }
    %entry;
}
1;
